package org.eclipse.jdt.internal.compiler.flow;

import java.util.Arrays;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;

public class UnconditionalFlowInfo extends FlowInfo
{
    public static final boolean COVERAGE_TEST_FLAG = false;
    public static int CoverageTestId;
    public long definiteInits;
    public long potentialInits;
    public long nullBit1;
    public long nullBit2;
    public long nullBit3;
    public long nullBit4;
    public long iNBit;
    public long iNNBit;
    public static final int extraLength = 8;
    public long[][] extra;
    public int maxFieldCount;
    public static final int BitCacheSize = 64;
    public static final int IN = 6;
    public static final int INN = 7;
    
    public UnconditionalFlowInfo() {
    }
    
    public static UnconditionalFlowInfo fakeInitializedFlowInfo(int localsCount, int maxFieldCount) {
        UnconditionalFlowInfo flowInfo = new UnconditionalFlowInfo();
        flowInfo.maxFieldCount = maxFieldCount;
        for (int i = 0; i < localsCount; ++i) {
            flowInfo.markAsDefinitelyAssigned(i + maxFieldCount);
        }
        return flowInfo;
    }
    
    @Override
    public FlowInfo addInitializationsFrom(FlowInfo inits) {
        return this.addInfoFrom(inits, true);
    }
    
    @Override
    public FlowInfo addNullInfoFrom(FlowInfo inits) {
        return this.addInfoFrom(inits, false);
    }
    
    private FlowInfo addInfoFrom(FlowInfo inits, boolean handleInits) {
        if (this == UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        if (inits == UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        UnconditionalFlowInfo otherInits = inits.unconditionalInits();
        if (handleInits) {
            this.definiteInits |= otherInits.definiteInits;
            this.potentialInits |= otherInits.potentialInits;
        }
        boolean thisHadNulls = (this.tagBits & 0x4) != 0x0;
        boolean otherHasNulls = (otherInits.tagBits & 0x4) != 0x0;
        if (otherHasNulls) {
            if (!thisHadNulls) {
                this.nullBit1 = otherInits.nullBit1;
                this.nullBit2 = otherInits.nullBit2;
                this.nullBit3 = otherInits.nullBit3;
                this.nullBit4 = otherInits.nullBit4;
                this.iNBit = otherInits.iNBit;
                this.iNNBit = otherInits.iNNBit;
            }
            else {
                long a1 = this.nullBit1;
                long a2 = this.nullBit2;
                long a3 = this.nullBit3;
                long a4 = this.nullBit4;
                long protNN1111 = a1 & a2 & a3 & a4;
                long acceptNonNull = otherInits.iNNBit;
                long acceptNull = otherInits.iNBit | protNN1111;
                long dontResetToStart = ~protNN1111 | acceptNonNull;
                a1 &= dontResetToStart;
                a2 &= (dontResetToStart & acceptNull);
                a3 &= (dontResetToStart & acceptNonNull);
                a4 &= dontResetToStart;
                a1 &= (a2 | a3 | a4);
                long b1;
                long b2;
                long nb2;
                long b3;
                long nb3;
                long na4;
                long na5;
                long na6;
                long b4;
                long nb4;
                this.nullBit1 = ((b1 = otherInits.nullBit1) | (a1 & ((a3 & a4 & (nb2 = ~(b2 = otherInits.nullBit2)) & (nb3 = ~(b3 = otherInits.nullBit4))) | (((na4 = ~a4) | (na5 = ~a3)) & (((na6 = ~a2) & nb2) | (a2 & (nb4 = ~(b4 = otherInits.nullBit3)) & nb3))))));
                long nb5;
                long na7;
                this.nullBit2 = ((b2 & (nb3 | nb4)) | (na5 & na4 & b2) | (a2 & ((nb4 & nb3) | ((nb5 = ~b1) & (na5 | (na7 = ~a1))) | (a1 & b2))));
                this.nullBit3 = ((b4 & ((nb5 & (b2 | a2 | na7)) | (b1 & (b3 | nb2 | (a1 & a3))) | (na7 & na6 & na4))) | (a3 & nb2 & nb3) | (nb5 & ((((na6 & a4) | na7) & a3) | (a1 & na6 & na4 & b2))));
                this.nullBit4 = ((nb5 & ((a4 & ((na5 & nb4) | ((a3 | na6) & nb2))) | (a1 & ((a3 & nb2 & b3) | (a2 & b2 & (b3 | (a3 & na4 & nb4))))))) | (b1 & ((a3 & a4 & b3) | (na6 & na4 & nb4 & b3) | (a2 & (((b4 | a4) & b3) | (na5 & a4 & b2 & b4))) | (na7 & (b3 | ((a4 | a2) & b2 & b4))))) | (((na7 & ((na5 & nb4) | (na6 & nb2))) | (a1 & ((nb2 & nb4) | (a2 & a3)))) & b3));
                this.iNBit &= otherInits.iNBit;
                this.iNNBit &= otherInits.iNNBit;
            }
            this.tagBits |= 0x4;
        }
        if (this.extra != null || otherInits.extra != null) {
            int mergeLimit = 0;
            int copyLimit = 0;
            if (this.extra != null) {
                if (otherInits.extra != null) {
                    int length;
                    int otherLength2;
                    if ((length = this.extra[0].length) < (otherLength2 = otherInits.extra[0].length)) {
                        for (int i = 0; i < 8; ++i) {
                            System.arraycopy(this.extra[i], 0, this.extra[i] = new long[otherLength2], 0, length);
                        }
                        mergeLimit = length;
                        copyLimit = otherLength2;
                    }
                    else {
                        mergeLimit = otherLength2;
                    }
                }
            }
            else if (otherInits.extra != null) {
                this.extra = new long[8][];
                int otherLength;
                System.arraycopy(otherInits.extra[0], 0, this.extra[0] = new long[otherLength = otherInits.extra[0].length], 0, otherLength);
                System.arraycopy(otherInits.extra[1], 0, this.extra[1] = new long[otherLength], 0, otherLength);
                if (otherHasNulls) {
                    for (int j = 2; j < 8; ++j) {
                        System.arraycopy(otherInits.extra[j], 0, this.extra[j] = new long[otherLength], 0, otherLength);
                    }
                }
                else {
                    for (int j = 2; j < 8; ++j) {
                        this.extra[j] = new long[otherLength];
                    }
                    System.arraycopy(otherInits.extra[6], 0, this.extra[6], 0, otherLength);
                    System.arraycopy(otherInits.extra[7], 0, this.extra[7], 0, otherLength);
                }
            }
            if (handleInits) {
                int k;
                for (k = 0; k < mergeLimit; ++k) {
                    long[] array = this.extra[0];
                    int n = k;
                    array[n] |= otherInits.extra[0][k];
                    long[] array2 = this.extra[1];
                    int n2 = k;
                    array2[n2] |= otherInits.extra[1][k];
                }
                while (k < copyLimit) {
                    this.extra[0][k] = otherInits.extra[0][k];
                    this.extra[1][k] = otherInits.extra[1][k];
                    ++k;
                }
            }
            if (!thisHadNulls) {
                if (copyLimit < mergeLimit) {
                    copyLimit = mergeLimit;
                }
                mergeLimit = 0;
            }
            if (!otherHasNulls) {
                copyLimit = 0;
                mergeLimit = 0;
            }
            int k;
            for (k = 0; k < mergeLimit; ++k) {
                long a1 = this.extra[2][k];
                long a2 = this.extra[3][k];
                long a3 = this.extra[4][k];
                long a4 = this.extra[5][k];
                long protNN1112 = a1 & a2 & a3 & a4;
                long acceptNonNull2 = otherInits.extra[7][k];
                long acceptNull2 = otherInits.extra[6][k] | protNN1112;
                long dontResetToStart2 = ~protNN1112 | acceptNonNull2;
                a1 &= dontResetToStart2;
                a2 &= (dontResetToStart2 & acceptNull2);
                a3 &= (dontResetToStart2 & acceptNonNull2);
                a4 &= dontResetToStart2;
                a1 &= (a2 | a3 | a4);
                long b1;
                long b2;
                long nb2;
                long b3;
                long nb3;
                long na4;
                long na5;
                long na6;
                long b4;
                long nb4;
                this.extra[2][k] = ((b1 = otherInits.extra[2][k]) | (a1 & ((a3 & a4 & (nb2 = ~(b2 = otherInits.extra[3][k])) & (nb3 = ~(b3 = otherInits.extra[5][k]))) | (((na4 = ~a4) | (na5 = ~a3)) & (((na6 = ~a2) & nb2) | (a2 & (nb4 = ~(b4 = otherInits.extra[4][k])) & nb3))))));
                long nb5;
                long na7;
                this.extra[3][k] = ((b2 & (nb3 | nb4)) | (na5 & na4 & b2) | (a2 & ((nb4 & nb3) | ((nb5 = ~b1) & (na5 | (na7 = ~a1))) | (a1 & b2))));
                this.extra[4][k] = ((b4 & ((nb5 & (b2 | a2 | na7)) | (b1 & (b3 | nb2 | (a1 & a3))) | (na7 & na6 & na4))) | (a3 & nb2 & nb3) | (nb5 & ((((na6 & a4) | na7) & a3) | (a1 & na6 & na4 & b2))));
                this.extra[5][k] = ((nb5 & ((a4 & ((na5 & nb4) | ((a3 | na6) & nb2))) | (a1 & ((a3 & nb2 & b3) | (a2 & b2 & (b3 | (a3 & na4 & nb4))))))) | (b1 & ((a3 & a4 & b3) | (na6 & na4 & nb4 & b3) | (a2 & (((b4 | a4) & b3) | (na5 & a4 & b2 & b4))) | (na7 & (b3 | ((a4 | a2) & b2 & b4))))) | (((na7 & ((na5 & nb4) | (na6 & nb2))) | (a1 & ((nb2 & nb4) | (a2 & a3)))) & b3));
                long[] array3 = this.extra[6];
                int n3 = k;
                array3[n3] &= otherInits.extra[6][k];
                long[] array4 = this.extra[7];
                int n4 = k;
                array4[n4] &= otherInits.extra[7][k];
            }
            while (k < copyLimit) {
                for (int j = 2; j < 8; ++j) {
                    this.extra[j][k] = otherInits.extra[j][k];
                }
                ++k;
            }
            return this;
        }
        else {
            return this;
        }
    }
    
    @Override
    public FlowInfo addPotentialInitializationsFrom(FlowInfo inits) {
        if (this == UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        if (inits == UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        UnconditionalFlowInfo otherInits = inits.unconditionalInits();
        this.potentialInits |= otherInits.potentialInits;
        if (this.extra != null) {
            if (otherInits.extra != null) {
                int i = 0;
                int length;
                int otherLength2;
                if ((length = this.extra[0].length) < (otherLength2 = otherInits.extra[0].length)) {
                    for (int j = 0; j < 8; ++j) {
                        System.arraycopy(this.extra[j], 0, this.extra[j] = new long[otherLength2], 0, length);
                    }
                    while (i < length) {
                        long[] array2 = this.extra[1];
                        int n2 = i;
                        array2[n2] |= otherInits.extra[1][i];
                        ++i;
                    }
                    while (i < otherLength2) {
                        this.extra[1][i] = otherInits.extra[1][i];
                        ++i;
                    }
                }
                else {
                    while (i < otherLength2) {
                        long[] array = this.extra[1];
                        int n = i;
                        array[n] |= otherInits.extra[1][i];
                        ++i;
                    }
                }
            }
        }
        else if (otherInits.extra != null) {
            int otherLength = otherInits.extra[0].length;
            this.createExtraSpace(otherLength);
            System.arraycopy(otherInits.extra[1], 0, this.extra[1], 0, otherLength);
        }
        this.addPotentialNullInfoFrom(otherInits);
        return this;
    }
    
    public UnconditionalFlowInfo addPotentialNullInfoFrom(UnconditionalFlowInfo otherInits) {
        if ((this.tagBits & 0x3) != 0x0 || (otherInits.tagBits & 0x3) != 0x0 || (otherInits.tagBits & 0x4) == 0x0) {
            return this;
        }
        boolean thisHadNulls = (this.tagBits & 0x4) != 0x0;
        boolean thisHasNulls = false;
        if (thisHadNulls) {
            long b2;
            long b3;
            long nb3;
            long b4;
            long nb5;
            long b5;
            long a1;
            long a2;
            long a3;
            long nb6;
            long a4;
            long na2;
            long na3;
            long na4;
            this.nullBit1 = ((a1 = this.nullBit1) & (((a2 = this.nullBit3) & (a3 = this.nullBit4) & (((nb5 = ~(b2 = otherInits.nullBit2)) & (nb6 = ~(b5 = otherInits.nullBit4))) | ((b4 = otherInits.nullBit1) & (b3 = otherInits.nullBit3)))) | ((na2 = ~(a4 = this.nullBit2)) & ((b4 & b3) | (((na3 = ~a3) | (na4 = ~a2)) & nb5))) | (a4 & ((na3 | na4) & (((nb3 = ~b3) & nb6) | (b4 & b2))))));
            long nb4;
            long na5;
            this.nullBit2 = ((b2 & (nb3 | (nb4 = ~b4))) | (a4 & ((nb3 & nb6) | b2 | na4 | (na5 = ~a1))));
            this.nullBit3 = ((b3 & ((nb4 & b2) | (a4 & (nb5 | a2)) | (na5 & nb5) | (a1 & na2 & na3 & b4))) | (a2 & ((nb5 & nb6) | (na2 & a3) | na5)) | (a1 & na2 & na3 & b2));
            this.nullBit4 = ((na4 & ((nb4 & nb3 & b5) | (a3 & (nb3 | (b4 & b2))))) | (nb5 & ((na4 & b4 & nb3) | (na2 & ((nb4 & b5) | (b4 & nb3) | a3)))) | (a2 & ((a3 & (nb5 | (b4 & b3))) | (a1 & a4 & ((nb4 & b5) | (na3 & (b2 | b4) & nb3))))));
            if ((this.nullBit2 | this.nullBit3 | this.nullBit4) != 0x0L) {
                thisHasNulls = true;
            }
        }
        else {
            this.nullBit1 = 0L;
            long b2;
            long n = b2 = otherInits.nullBit2;
            long b3;
            long nb3 = ~(b3 = otherInits.nullBit3);
            long b4;
            long nb4;
            this.nullBit2 = (n & (nb3 | (nb4 = ~(b4 = otherInits.nullBit1))));
            long nb5;
            this.nullBit3 = (b3 & (nb4 | (nb5 = ~b2)));
            long b5;
            this.nullBit4 = ((~b4 & ~b3 & (b5 = otherInits.nullBit4)) | (~b2 & ((b4 & ~b3) | (~b4 & b5))));
            if ((this.nullBit2 | this.nullBit3 | this.nullBit4) != 0x0L) {
                thisHasNulls = true;
            }
        }
        if (otherInits.extra != null) {
            int mergeLimit = 0;
            int copyLimit = otherInits.extra[0].length;
            if (this.extra == null) {
                this.createExtraSpace(copyLimit);
            }
            else {
                mergeLimit = copyLimit;
                if (mergeLimit > this.extra[0].length) {
                    mergeLimit = this.extra[0].length;
                    for (int j = 0; j < 8; ++j) {
                        System.arraycopy(this.extra[j], 0, this.extra[j] = new long[copyLimit], 0, mergeLimit);
                    }
                    if (!thisHadNulls) {
                        mergeLimit = 0;
                    }
                }
            }
            int i;
            for (i = 0; i < mergeLimit; ++i) {
                long b2;
                long b3;
                long nb3;
                long b4;
                long nb5;
                long b5;
                long a1;
                long a2;
                long a3;
                long nb6;
                long a4;
                long na2;
                long na3;
                long na4;
                this.extra[2][i] = ((a1 = this.extra[2][i]) & (((a2 = this.extra[4][i]) & (a3 = this.extra[5][i]) & (((nb5 = ~(b2 = otherInits.extra[3][i])) & (nb6 = ~(b5 = otherInits.extra[5][i]))) | ((b4 = otherInits.extra[2][i]) & (b3 = otherInits.extra[4][i])))) | ((na2 = ~(a4 = this.extra[3][i])) & ((b4 & b3) | (((na3 = ~a3) | (na4 = ~a2)) & nb5))) | (a4 & ((na3 | na4) & (((nb3 = ~b3) & nb6) | (b4 & b2))))));
                long nb4;
                long na5;
                this.extra[3][i] = ((b2 & (nb3 | (nb4 = ~b4))) | (a4 & ((nb3 & nb6) | b2 | na4 | (na5 = ~a1))));
                this.extra[4][i] = ((b3 & ((nb4 & b2) | (a4 & (nb5 | a2)) | (na5 & nb5) | (a1 & na2 & na3 & b4))) | (a2 & ((nb5 & nb6) | (na2 & a3) | na5)) | (a1 & na2 & na3 & b2));
                this.extra[5][i] = ((na4 & ((nb4 & nb3 & b5) | (a3 & (nb3 | (b4 & b2))))) | (nb5 & ((na4 & b4 & nb3) | (na2 & ((nb4 & b5) | (b4 & nb3) | a3)))) | (a2 & ((a3 & (nb5 | (b4 & b3))) | (a1 & a4 & ((nb4 & b5) | (na3 & (b2 | b4) & nb3))))));
                if ((this.extra[3][i] | this.extra[4][i] | this.extra[5][i]) != 0x0L) {
                    thisHasNulls = true;
                }
            }
            while (i < copyLimit) {
                this.extra[2][i] = 0L;
                long[] array = this.extra[3];
                int n2 = i;
                long b2;
                long n3 = b2 = otherInits.extra[3][i];
                long b3;
                long nb3 = ~(b3 = otherInits.extra[4][i]);
                long b4;
                long nb4;
                array[n2] = (n3 & (nb3 | (nb4 = ~(b4 = otherInits.extra[2][i]))));
                long nb5;
                this.extra[4][i] = (b3 & (nb4 | (nb5 = ~b2)));
                long b5;
                this.extra[5][i] = ((~b4 & ~b3 & (b5 = otherInits.extra[5][i])) | (~b2 & ((b4 & ~b3) | (~b4 & b5))));
                if ((this.extra[3][i] | this.extra[4][i] | this.extra[5][i]) != 0x0L) {
                    thisHasNulls = true;
                }
                ++i;
            }
        }
        if (thisHasNulls) {
            this.tagBits |= 0x4;
            return this;
        }
        this.tagBits &= 0x4;
        return this;
    }
    
    @Override
    public final boolean cannotBeDefinitelyNullOrNonNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x4) == 0x0 || (local.type.tagBits & 0x2L) != 0x0L) {
            return false;
        }
        int position;
        if ((position = local.id + this.maxFieldCount) < 64) {
            return (((~this.nullBit1 & ((this.nullBit2 & this.nullBit3) | this.nullBit4)) | (~this.nullBit2 & ~this.nullBit3 & this.nullBit4)) & 1L << position) != 0x0L;
        }
        if (this.extra == null) {
            return false;
        }
        int vectorIndex;
        if ((vectorIndex = position / 64 - 1) >= this.extra[0].length) {
            return false;
        }
        long n = ~this.extra[2][vectorIndex];
        long a2 = this.extra[3][vectorIndex];
        long a3;
        long a4;
        return (((n & ((a2 & (a3 = this.extra[4][vectorIndex])) | (a4 = this.extra[5][vectorIndex]))) | (~a2 & ~a3 & a4)) & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean cannotBeNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x4) == 0x0 || (local.type.tagBits & 0x2L) != 0x0L) {
            return false;
        }
        int position;
        if ((position = local.id + this.maxFieldCount) < 64) {
            return (this.nullBit1 & this.nullBit3 & ((this.nullBit2 & this.nullBit4) | ~this.nullBit2) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[0].length && (this.extra[2][vectorIndex] & this.extra[4][vectorIndex] & ((this.extra[3][vectorIndex] & this.extra[5][vectorIndex]) | ~this.extra[3][vectorIndex]) & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean canOnlyBeNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x4) == 0x0 || (local.type.tagBits & 0x2L) != 0x0L) {
            return false;
        }
        int position;
        if ((position = local.id + this.maxFieldCount) < 64) {
            return (this.nullBit1 & this.nullBit2 & (~this.nullBit3 | ~this.nullBit4) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[0].length && (this.extra[2][vectorIndex] & this.extra[3][vectorIndex] & (~this.extra[4][vectorIndex] | ~this.extra[5][vectorIndex]) & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public FlowInfo copy() {
        if (this == UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        UnconditionalFlowInfo copy = new UnconditionalFlowInfo();
        copy.definiteInits = this.definiteInits;
        copy.potentialInits = this.potentialInits;
        boolean hasNullInfo = (this.tagBits & 0x4) != 0x0;
        if (hasNullInfo) {
            copy.nullBit1 = this.nullBit1;
            copy.nullBit2 = this.nullBit2;
            copy.nullBit3 = this.nullBit3;
            copy.nullBit4 = this.nullBit4;
        }
        copy.iNBit = this.iNBit;
        copy.iNNBit = this.iNNBit;
        copy.tagBits = this.tagBits;
        copy.maxFieldCount = this.maxFieldCount;
        if (this.extra != null) {
            copy.extra = new long[8][];
            int length;
            System.arraycopy(this.extra[0], 0, copy.extra[0] = new long[length = this.extra[0].length], 0, length);
            System.arraycopy(this.extra[1], 0, copy.extra[1] = new long[length], 0, length);
            if (hasNullInfo) {
                for (int j = 2; j < 8; ++j) {
                    System.arraycopy(this.extra[j], 0, copy.extra[j] = new long[length], 0, length);
                }
            }
            else {
                for (int j = 2; j < 8; ++j) {
                    copy.extra[j] = new long[length];
                }
            }
        }
        return copy;
    }
    
    public UnconditionalFlowInfo discardInitializationInfo() {
        if (this == UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        long n = 0L;
        this.potentialInits = n;
        this.definiteInits = n;
        if (this.extra != null) {
            for (int i = 0, length = this.extra[0].length; i < length; ++i) {
                this.extra[0][i] = (this.extra[1][i] = 0L);
            }
            return this;
        }
        return this;
    }
    
    public UnconditionalFlowInfo discardNonFieldInitializations() {
        int limit = this.maxFieldCount;
        if (limit < 64) {
            long mask = (1L << limit) - 1L;
            this.definiteInits &= mask;
            this.potentialInits &= mask;
            this.nullBit1 &= mask;
            this.nullBit2 &= mask;
            this.nullBit3 &= mask;
            this.nullBit4 &= mask;
            this.iNBit &= mask;
            this.iNNBit &= mask;
        }
        if (this.extra == null) {
            return this;
        }
        int length = this.extra[0].length;
        int vectorIndex;
        if ((vectorIndex = limit / 64 - 1) >= length) {
            return this;
        }
        if (vectorIndex >= 0) {
            long mask2 = (1L << limit % 64) - 1L;
            for (int j = 0; j < 8; ++j) {
                long[] array = this.extra[j];
                int n = vectorIndex;
                array[n] &= mask2;
            }
        }
        for (int i = vectorIndex + 1; i < length; ++i) {
            for (int k = 0; k < 8; ++k) {
                this.extra[k][i] = 0L;
            }
        }
        return this;
    }
    
    @Override
    public FlowInfo initsWhenFalse() {
        return this;
    }
    
    @Override
    public FlowInfo initsWhenTrue() {
        return this;
    }
    
    private final boolean isDefinitelyAssigned(int position) {
        if (position < 64) {
            return (this.definiteInits & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[0].length && (this.extra[0][vectorIndex] & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean isDefinitelyAssigned(FieldBinding field) {
        return (this.tagBits & 0x1) != 0x0 || this.isDefinitelyAssigned(field.id);
    }
    
    @Override
    public final boolean isDefinitelyAssigned(LocalVariableBinding local) {
        return ((this.tagBits & 0x1) != 0x0 && (local.declaration.bits & 0x40000000) != 0x0) || this.isDefinitelyAssigned(local.id + this.maxFieldCount);
    }
    
    @Override
    public final boolean isDefinitelyNonNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x3) != 0x0 || (this.tagBits & 0x4) == 0x0) {
            return false;
        }
        if ((local.type.tagBits & 0x2L) != 0x0L || local.constant() != Constant.NotAConstant) {
            return true;
        }
        int position = local.id + this.maxFieldCount;
        if (position < 64) {
            return (this.nullBit1 & this.nullBit3 & (~this.nullBit2 | this.nullBit4) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[2].length && (this.extra[2][vectorIndex] & this.extra[4][vectorIndex] & (~this.extra[3][vectorIndex] | this.extra[5][vectorIndex]) & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean isDefinitelyNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x3) != 0x0 || (this.tagBits & 0x4) == 0x0 || (local.type.tagBits & 0x2L) != 0x0L) {
            return false;
        }
        int position = local.id + this.maxFieldCount;
        if (position < 64) {
            return (this.nullBit1 & this.nullBit2 & (~this.nullBit3 | ~this.nullBit4) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[2].length && (this.extra[2][vectorIndex] & this.extra[3][vectorIndex] & (~this.extra[4][vectorIndex] | ~this.extra[5][vectorIndex]) & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean isDefinitelyUnknown(LocalVariableBinding local) {
        if ((this.tagBits & 0x3) != 0x0 || (this.tagBits & 0x4) == 0x0) {
            return false;
        }
        int position = local.id + this.maxFieldCount;
        if (position < 64) {
            return (this.nullBit1 & this.nullBit4 & ~this.nullBit2 & ~this.nullBit3 & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[2].length && (this.extra[2][vectorIndex] & this.extra[5][vectorIndex] & ~this.extra[3][vectorIndex] & ~this.extra[4][vectorIndex] & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean hasNullInfoFor(LocalVariableBinding local) {
        if ((this.tagBits & 0x3) != 0x0 || (this.tagBits & 0x4) == 0x0) {
            return false;
        }
        int position = local.id + this.maxFieldCount;
        if (position < 64) {
            return ((this.nullBit1 | this.nullBit2 | this.nullBit3 | this.nullBit4) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[2].length && ((this.extra[2][vectorIndex] | this.extra[3][vectorIndex] | this.extra[4][vectorIndex] | this.extra[5][vectorIndex]) & 1L << position % 64) != 0x0L;
    }
    
    private final boolean isPotentiallyAssigned(int position) {
        if (position < 64) {
            return (this.potentialInits & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[0].length && (this.extra[1][vectorIndex] & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean isPotentiallyAssigned(FieldBinding field) {
        return this.isPotentiallyAssigned(field.id);
    }
    
    @Override
    public final boolean isPotentiallyAssigned(LocalVariableBinding local) {
        return local.constant() != Constant.NotAConstant || this.isPotentiallyAssigned(local.id + this.maxFieldCount);
    }
    
    @Override
    public final boolean isPotentiallyNonNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x4) == 0x0 || (local.type.tagBits & 0x2L) != 0x0L) {
            return false;
        }
        int position;
        if ((position = local.id + this.maxFieldCount) < 64) {
            return (this.nullBit3 & (~this.nullBit1 | ~this.nullBit2) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[2].length && (this.extra[4][vectorIndex] & (~this.extra[2][vectorIndex] | ~this.extra[3][vectorIndex]) & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean isPotentiallyNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x4) == 0x0 || (local.type.tagBits & 0x2L) != 0x0L) {
            return false;
        }
        int position;
        if ((position = local.id + this.maxFieldCount) < 64) {
            return (this.nullBit2 & (~this.nullBit1 | ~this.nullBit3) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[2].length && (this.extra[3][vectorIndex] & (~this.extra[2][vectorIndex] | ~this.extra[4][vectorIndex]) & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean isPotentiallyUnknown(LocalVariableBinding local) {
        if ((this.tagBits & 0x3) != 0x0 || (this.tagBits & 0x4) == 0x0) {
            return false;
        }
        int position = local.id + this.maxFieldCount;
        if (position < 64) {
            return (this.nullBit4 & (~this.nullBit1 | (~this.nullBit2 & ~this.nullBit3)) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[2].length && (this.extra[5][vectorIndex] & (~this.extra[2][vectorIndex] | (~this.extra[3][vectorIndex] & ~this.extra[4][vectorIndex])) & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean isProtectedNonNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x4) == 0x0 || (local.type.tagBits & 0x2L) != 0x0L) {
            return false;
        }
        int position;
        if ((position = local.id + this.maxFieldCount) < 64) {
            return (this.nullBit1 & this.nullBit3 & this.nullBit4 & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[0].length && (this.extra[2][vectorIndex] & this.extra[4][vectorIndex] & this.extra[5][vectorIndex] & 1L << position % 64) != 0x0L;
    }
    
    @Override
    public final boolean isProtectedNull(LocalVariableBinding local) {
        if ((this.tagBits & 0x4) == 0x0 || (local.type.tagBits & 0x2L) != 0x0L) {
            return false;
        }
        int position;
        if ((position = local.id + this.maxFieldCount) < 64) {
            return (this.nullBit1 & this.nullBit2 & (this.nullBit3 ^ this.nullBit4) & 1L << position) != 0x0L;
        }
        int vectorIndex;
        return this.extra != null && (vectorIndex = position / 64 - 1) < this.extra[0].length && (this.extra[2][vectorIndex] & this.extra[3][vectorIndex] & (this.extra[4][vectorIndex] ^ this.extra[5][vectorIndex]) & 1L << position % 64) != 0x0L;
    }
    
    protected static boolean isTrue(boolean expression, String message) {
        if (!expression) {
            throw new UnconditionalFlowInfo.AssertionFailedException("assertion failed: " + message);
        }
        return expression;
    }
    
    @Override
    public void markAsComparedEqualToNonNull(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask;
                long a1;
                long a2;
                long na2;
                long a3;
                long a4;
                if (((mask = 1L << position) & (a1 = this.nullBit1) & (na2 = ~(a2 = this.nullBit2)) & ~(a3 = this.nullBit3) & (a4 = this.nullBit4)) != 0x0L) {
                    this.nullBit4 &= ~mask;
                }
                else if ((mask & a1 & na2 & a3) == 0x0L) {
                    this.nullBit4 |= mask;
                    if ((mask & a1) == 0x0L) {
                        if ((mask & a2 & (a3 ^ a4)) != 0x0L) {
                            this.nullBit2 &= ~mask;
                        }
                        else if ((mask & (a2 | a3 | a4)) == 0x0L) {
                            this.nullBit2 |= mask;
                        }
                    }
                }
                this.nullBit1 |= mask;
                this.nullBit3 |= mask;
                this.iNBit &= ~mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        int newLength = vectorIndex + 1;
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[newLength], 0, oldLength);
                        }
                    }
                }
                long mask;
                long a1;
                long a2;
                long na2;
                long a3;
                long a4;
                if (((mask = 1L << position % 64) & (a1 = this.extra[2][vectorIndex]) & (na2 = ~(a2 = this.extra[3][vectorIndex])) & ~(a3 = this.extra[4][vectorIndex]) & (a4 = this.extra[5][vectorIndex])) != 0x0L) {
                    long[] array4 = this.extra[5];
                    int n4 = vectorIndex;
                    array4[n4] &= ~mask;
                }
                else if ((mask & a1 & na2 & a3) == 0x0L) {
                    long[] array = this.extra[5];
                    int n = vectorIndex;
                    array[n] |= mask;
                    if ((mask & a1) == 0x0L) {
                        if ((mask & a2 & (a3 ^ a4)) != 0x0L) {
                            long[] array3 = this.extra[3];
                            int n3 = vectorIndex;
                            array3[n3] &= ~mask;
                        }
                        else if ((mask & (a2 | a3 | a4)) == 0x0L) {
                            long[] array2 = this.extra[3];
                            int n2 = vectorIndex;
                            array2[n2] |= mask;
                        }
                    }
                }
                long[] array5 = this.extra[2];
                int n5 = vectorIndex;
                array5[n5] |= mask;
                long[] array6 = this.extra[4];
                int n6 = vectorIndex;
                array6[n6] |= mask;
                long[] array7 = this.extra[6];
                int n7 = vectorIndex;
                array7[n7] &= ~mask;
            }
        }
    }
    
    @Override
    public void markAsComparedEqualToNull(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask;
                if (((mask = 1L << position) & this.nullBit1) != 0x0L) {
                    if ((mask & (~this.nullBit2 | this.nullBit3 | ~this.nullBit4)) != 0x0L) {
                        this.nullBit4 &= ~mask;
                    }
                }
                else if ((mask & this.nullBit4) != 0x0L) {
                    this.nullBit3 &= ~mask;
                }
                else if ((mask & this.nullBit2) != 0x0L) {
                    this.nullBit3 &= ~mask;
                    this.nullBit4 |= mask;
                }
                else {
                    this.nullBit3 |= mask;
                }
                this.nullBit1 |= mask;
                this.nullBit2 |= mask;
                this.iNNBit &= ~mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                long mask = 1L << position % 64;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        int newLength = vectorIndex + 1;
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[newLength], 0, oldLength);
                        }
                    }
                }
                if ((mask & this.extra[2][vectorIndex]) != 0x0L) {
                    if ((mask & (~this.extra[3][vectorIndex] | this.extra[4][vectorIndex] | ~this.extra[5][vectorIndex])) != 0x0L) {
                        long[] array5 = this.extra[5];
                        int n5 = vectorIndex;
                        array5[n5] &= ~mask;
                    }
                }
                else if ((mask & this.extra[5][vectorIndex]) != 0x0L) {
                    long[] array4 = this.extra[4];
                    int n4 = vectorIndex;
                    array4[n4] &= ~mask;
                }
                else if ((mask & this.extra[3][vectorIndex]) != 0x0L) {
                    long[] array2 = this.extra[4];
                    int n2 = vectorIndex;
                    array2[n2] &= ~mask;
                    long[] array3 = this.extra[5];
                    int n3 = vectorIndex;
                    array3[n3] |= mask;
                }
                else {
                    long[] array = this.extra[4];
                    int n = vectorIndex;
                    array[n] |= mask;
                }
                long[] array6 = this.extra[2];
                int n6 = vectorIndex;
                array6[n6] |= mask;
                long[] array7 = this.extra[3];
                int n7 = vectorIndex;
                array7[n7] |= mask;
                long[] array8 = this.extra[7];
                int n8 = vectorIndex;
                array8[n8] &= ~mask;
            }
        }
    }
    
    private final void markAsDefinitelyAssigned(int position) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            if (position < 64) {
                long mask2;
                this.definiteInits |= (mask2 = 1L << position);
                this.potentialInits |= mask2;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[vectorIndex + 1], 0, oldLength);
                        }
                    }
                }
                long[] array = this.extra[0];
                int n = vectorIndex;
                long mask;
                array[n] |= (mask = 1L << position % 64);
                long[] array2 = this.extra[1];
                int n2 = vectorIndex;
                array2[n2] |= mask;
            }
        }
    }
    
    @Override
    public void markAsDefinitelyAssigned(FieldBinding field) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.markAsDefinitelyAssigned(field.id);
        }
    }
    
    @Override
    public void markAsDefinitelyAssigned(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.markAsDefinitelyAssigned(local.id + this.maxFieldCount);
        }
    }
    
    @Override
    public void markAsDefinitelyNonNull(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask;
                this.nullBit1 |= (mask = 1L << position);
                this.nullBit3 |= mask;
                this.nullBit2 &= (mask ^= -1L);
                this.nullBit4 &= mask;
                this.iNBit &= mask;
                this.iNNBit &= mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[vectorIndex + 1], 0, oldLength);
                        }
                    }
                }
                long[] array = this.extra[2];
                int n = vectorIndex;
                long mask;
                array[n] |= (mask = 1L << position % 64);
                long[] array2 = this.extra[4];
                int n2 = vectorIndex;
                array2[n2] |= mask;
                long[] array3 = this.extra[3];
                int n3 = vectorIndex;
                array3[n3] &= (mask ^= -1L);
                long[] array4 = this.extra[5];
                int n4 = vectorIndex;
                array4[n4] &= mask;
                long[] array5 = this.extra[6];
                int n5 = vectorIndex;
                array5[n5] &= mask;
                long[] array6 = this.extra[7];
                int n6 = vectorIndex;
                array6[n6] &= mask;
            }
        }
    }
    
    @Override
    public void markAsDefinitelyNull(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask;
                this.nullBit1 |= (mask = 1L << position);
                this.nullBit2 |= mask;
                this.nullBit3 &= (mask ^= -1L);
                this.nullBit4 &= mask;
                this.iNBit &= mask;
                this.iNNBit &= mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[vectorIndex + 1], 0, oldLength);
                        }
                    }
                }
                long[] array = this.extra[2];
                int n = vectorIndex;
                long mask;
                array[n] |= (mask = 1L << position % 64);
                long[] array2 = this.extra[3];
                int n2 = vectorIndex;
                array2[n2] |= mask;
                long[] array3 = this.extra[4];
                int n3 = vectorIndex;
                array3[n3] &= (mask ^= -1L);
                long[] array4 = this.extra[5];
                int n4 = vectorIndex;
                array4[n4] &= mask;
                long[] array5 = this.extra[6];
                int n5 = vectorIndex;
                array5[n5] &= mask;
                long[] array6 = this.extra[7];
                int n6 = vectorIndex;
                array6[n6] &= mask;
            }
        }
    }
    
    @Override
    public void markAsDefinitelyUnknown(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask;
                this.nullBit1 |= (mask = 1L << position);
                this.nullBit4 |= mask;
                this.nullBit2 &= (mask ^= -1L);
                this.nullBit3 &= mask;
                this.iNBit &= mask;
                this.iNNBit &= mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[vectorIndex + 1], 0, oldLength);
                        }
                    }
                }
                long[] array = this.extra[2];
                int n = vectorIndex;
                long mask;
                array[n] |= (mask = 1L << position % 64);
                long[] array2 = this.extra[5];
                int n2 = vectorIndex;
                array2[n2] |= mask;
                long[] array3 = this.extra[3];
                int n3 = vectorIndex;
                array3[n3] &= (mask ^= -1L);
                long[] array4 = this.extra[4];
                int n4 = vectorIndex;
                array4[n4] &= mask;
                long[] array5 = this.extra[6];
                int n5 = vectorIndex;
                array5[n5] &= mask;
                long[] array6 = this.extra[7];
                int n6 = vectorIndex;
                array6[n6] &= mask;
            }
        }
    }
    
    @Override
    public void resetNullInfo(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask;
                this.nullBit1 &= (mask = ~(1L << position));
                this.nullBit2 &= mask;
                this.nullBit3 &= mask;
                this.nullBit4 &= mask;
                this.iNBit &= mask;
                this.iNNBit &= mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null || vectorIndex >= this.extra[2].length) {
                    return;
                }
                long[] array = this.extra[2];
                int n = vectorIndex;
                long mask;
                array[n] &= (mask = ~(1L << position % 64));
                long[] array2 = this.extra[3];
                int n2 = vectorIndex;
                array2[n2] &= mask;
                long[] array3 = this.extra[4];
                int n3 = vectorIndex;
                array3[n3] &= mask;
                long[] array4 = this.extra[5];
                int n4 = vectorIndex;
                array4[n4] &= mask;
                long[] array5 = this.extra[6];
                int n5 = vectorIndex;
                array5[n5] &= mask;
                long[] array6 = this.extra[7];
                int n6 = vectorIndex;
                array6[n6] &= mask;
            }
        }
    }
    
    @Override
    public void markPotentiallyUnknownBit(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask = 1L << position;
                isTrue((this.nullBit1 & mask) == 0x0L, "Adding 'unknown' mark in unexpected state");
                this.nullBit4 |= mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[vectorIndex + 1], 0, oldLength);
                        }
                    }
                }
                long mask = 1L << position % 64;
                isTrue((this.extra[2][vectorIndex] & mask) == 0x0L, "Adding 'unknown' mark in unexpected state");
                long[] array = this.extra[5];
                int n = vectorIndex;
                array[n] |= mask;
            }
        }
    }
    
    @Override
    public void markPotentiallyNullBit(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask = 1L << position;
                isTrue((this.nullBit1 & mask) == 0x0L, "Adding 'potentially null' mark in unexpected state");
                this.nullBit2 |= mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[vectorIndex + 1], 0, oldLength);
                        }
                    }
                }
                long mask = 1L << position % 64;
                long[] array = this.extra[3];
                int n = vectorIndex;
                array[n] |= mask;
                isTrue((this.extra[2][vectorIndex] & mask) == 0x0L, "Adding 'potentially null' mark in unexpected state");
            }
        }
    }
    
    @Override
    public void markPotentiallyNonNullBit(LocalVariableBinding local) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            this.tagBits |= 0x4;
            int position;
            if ((position = local.id + this.maxFieldCount) < 64) {
                long mask = 1L << position;
                isTrue((this.nullBit1 & mask) == 0x0L, "Adding 'potentially non-null' mark in unexpected state");
                this.nullBit3 |= mask;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null) {
                    int length = vectorIndex + 1;
                    this.createExtraSpace(length);
                }
                else {
                    int oldLength;
                    if (vectorIndex >= (oldLength = this.extra[0].length)) {
                        for (int j = 0; j < 8; ++j) {
                            System.arraycopy(this.extra[j], 0, this.extra[j] = new long[vectorIndex + 1], 0, oldLength);
                        }
                    }
                }
                long mask = 1L << position % 64;
                isTrue((this.extra[2][vectorIndex] & mask) == 0x0L, "Adding 'potentially non-null' mark in unexpected state");
                long[] array = this.extra[4];
                int n = vectorIndex;
                array[n] |= mask;
            }
        }
    }
    
    @Override
    public UnconditionalFlowInfo mergedWith(UnconditionalFlowInfo otherInits) {
        if ((otherInits.tagBits & 0x1) != 0x0 && this != UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        if ((this.tagBits & 0x1) != 0x0) {
            return (UnconditionalFlowInfo)otherInits.copy();
        }
        this.definiteInits &= otherInits.definiteInits;
        this.potentialInits |= otherInits.potentialInits;
        boolean thisHasNulls = (this.tagBits & 0x4) != 0x0;
        boolean otherHasNulls = (otherInits.tagBits & 0x4) != 0x0;
        boolean thisHadNulls = thisHasNulls;
        if ((otherInits.tagBits & 0x2) != 0x0) {
            otherHasNulls = false;
        }
        else if ((this.tagBits & 0x2) != 0x0) {
            this.nullBit1 = otherInits.nullBit1;
            this.nullBit2 = otherInits.nullBit2;
            this.nullBit3 = otherInits.nullBit3;
            this.nullBit4 = otherInits.nullBit4;
            this.iNBit = otherInits.iNBit;
            this.iNNBit = otherInits.iNNBit;
            thisHadNulls = false;
            thisHasNulls = otherHasNulls;
            this.tagBits = otherInits.tagBits;
        }
        else if (thisHadNulls) {
            if (otherHasNulls) {
                long b2;
                long b3;
                long b4;
                long nb5;
                long b5;
                long a1;
                long a2;
                long a3;
                long na3;
                long na5;
                long a4;
                this.nullBit1 = ((a1 = this.nullBit1) & (b4 = otherInits.nullBit1) & (((a2 = this.nullBit2) & (((b2 = otherInits.nullBit2) & ~(((a3 = this.nullBit3) & (a4 = this.nullBit4)) ^ ((b3 = otherInits.nullBit3) & (b5 = otherInits.nullBit4)))) | (a3 & a4 & (nb5 = ~b2)))) | ((na5 = ~a2) & ((b2 & b3 & b5) | (nb5 & ((na3 = ~a3) ^ b3))))));
                long nb4;
                long nb3;
                long na4;
                long nb6;
                long na6;
                this.nullBit2 = ((b2 & ((nb3 = ~b3) | (nb4 = ~b4) | (a3 & (a4 | (na4 = ~a1)) & (nb6 = ~b5)))) | (a2 & (b2 | ((na6 = ~a4) & b3 & (b5 | nb4)) | na3 | na4)));
                this.nullBit3 = ((a3 & (na4 | (a1 & na5) | (b3 & (na6 ^ b5)))) | (b3 & (nb4 | (b4 & nb5))));
                this.nullBit4 = ((na3 & ((nb4 & nb3 & b5) | (b4 & ((nb5 & nb3) | (a4 & b2 & nb6))) | (na4 & a4 & (nb3 | (b4 & b2))))) | (a3 & a4 & ((b3 & b5) | (b4 & nb5) | (na4 & a2))) | (na5 & ((nb4 & b5) | (b4 & nb3) | (na4 & a4)) & nb5) | (a1 & ((na3 & ((nb3 & b5) | (b4 & b2 & b3 & nb6) | (na5 & (nb3 | nb5)))) | (na5 & b3 & b5) | (a2 & ((nb4 & b5) | (a3 & na6 & b4)) & nb3))) | (nb4 & b2 & b3 & b5));
            }
            else {
                long a1 = this.nullBit1;
                this.nullBit1 = 0L;
                long a2 = this.nullBit2;
                long a3;
                long na4;
                long na3;
                this.nullBit2 = (a2 & (na3 = (~(a3 = this.nullBit3) | (na4 = ~a1))));
                long na5;
                long a4;
                this.nullBit3 = ((a3 & (((na5 = ~a2) & (a4 = this.nullBit4)) | na4)) | (a1 & na5 & ~a4));
                this.nullBit4 = (((na3 | na5) & na4 & a4) | (a1 & na3 & na5));
            }
            this.iNBit |= otherInits.iNBit;
            this.iNNBit |= otherInits.iNNBit;
        }
        else if (otherHasNulls) {
            this.nullBit1 = 0L;
            long b2 = otherInits.nullBit2;
            long b3;
            long b4;
            long nb4;
            long nb3;
            this.nullBit2 = (b2 & (nb3 = (~(b3 = otherInits.nullBit3) | (nb4 = ~(b4 = otherInits.nullBit1)))));
            long nb5;
            long b5;
            this.nullBit3 = ((b3 & (((nb5 = ~b2) & (b5 = otherInits.nullBit4)) | nb4)) | (b4 & nb5 & ~b5));
            this.nullBit4 = (((nb3 | nb5) & nb4 & b5) | (b4 & nb3 & nb5));
            this.iNBit |= otherInits.iNBit;
            this.iNNBit |= otherInits.iNNBit;
            thisHasNulls = (this.nullBit2 != 0L || this.nullBit3 != 0L || this.nullBit4 != 0L);
        }
        if (this.extra != null || otherInits.extra != null) {
            int mergeLimit = 0;
            int copyLimit = 0;
            int resetLimit = 0;
            if (this.extra != null) {
                if (otherInits.extra != null) {
                    int length;
                    int otherLength2;
                    if ((length = this.extra[0].length) < (otherLength2 = otherInits.extra[0].length)) {
                        for (int i = 0; i < 8; ++i) {
                            System.arraycopy(this.extra[i], 0, this.extra[i] = new long[otherLength2], 0, length);
                        }
                        mergeLimit = length;
                        copyLimit = otherLength2;
                    }
                    else {
                        mergeLimit = otherLength2;
                        resetLimit = length;
                    }
                }
                else {
                    resetLimit = this.extra[0].length;
                }
            }
            else if (otherInits.extra != null) {
                int otherLength = otherInits.extra[0].length;
                this.extra = new long[8][];
                for (int j = 0; j < 8; ++j) {
                    this.extra[j] = new long[otherLength];
                }
                System.arraycopy(otherInits.extra[1], 0, this.extra[1], 0, otherLength);
                System.arraycopy(otherInits.extra[6], 0, this.extra[6], 0, otherLength);
                System.arraycopy(otherInits.extra[7], 0, this.extra[7], 0, otherLength);
                copyLimit = otherLength;
            }
            int k;
            for (k = 0; k < mergeLimit; ++k) {
                long[] array = this.extra[0];
                int n = k;
                array[n] &= otherInits.extra[0][k];
                long[] array2 = this.extra[1];
                int n2 = k;
                array2[n2] |= otherInits.extra[1][k];
            }
            while (k < copyLimit) {
                this.extra[1][k] = otherInits.extra[1][k];
                ++k;
            }
            while (k < resetLimit) {
                this.extra[0][k] = 0L;
                ++k;
            }
            if (!otherHasNulls) {
                if (resetLimit < mergeLimit) {
                    resetLimit = mergeLimit;
                }
                copyLimit = 0;
                mergeLimit = 0;
            }
            if (!thisHadNulls) {
                resetLimit = 0;
            }
            for (k = 0; k < mergeLimit; ++k) {
                long b2;
                long b3;
                long b4;
                long nb5;
                long b5;
                long a1;
                long a2;
                long a3;
                long na3;
                long na5;
                long a4;
                this.extra[2][k] = ((a1 = this.extra[2][k]) & (b4 = otherInits.extra[2][k]) & (((a2 = this.extra[3][k]) & (((b2 = otherInits.extra[3][k]) & ~(((a3 = this.extra[4][k]) & (a4 = this.extra[5][k])) ^ ((b3 = otherInits.extra[4][k]) & (b5 = otherInits.extra[5][k])))) | (a3 & a4 & (nb5 = ~b2)))) | ((na5 = ~a2) & ((b2 & b3 & b5) | (nb5 & ((na3 = ~a3) ^ b3))))));
                long nb4;
                long nb3;
                long na4;
                long nb6;
                long na6;
                this.extra[3][k] = ((b2 & ((nb3 = ~b3) | (nb4 = ~b4) | (a3 & (a4 | (na4 = ~a1)) & (nb6 = ~b5)))) | (a2 & (b2 | ((na6 = ~a4) & b3 & (b5 | nb4)) | na3 | na4)));
                this.extra[4][k] = ((a3 & (na4 | (a1 & na5) | (b3 & (na6 ^ b5)))) | (b3 & (nb4 | (b4 & nb5))));
                this.extra[5][k] = ((na3 & ((nb4 & nb3 & b5) | (b4 & ((nb5 & nb3) | (a4 & b2 & nb6))) | (na4 & a4 & (nb3 | (b4 & b2))))) | (a3 & a4 & ((b3 & b5) | (b4 & nb5) | (na4 & a2))) | (na5 & ((nb4 & b5) | (b4 & nb3) | (na4 & a4)) & nb5) | (a1 & ((na3 & ((nb3 & b5) | (b4 & b2 & b3 & nb6) | (na5 & (nb3 | nb5)))) | (na5 & b3 & b5) | (a2 & ((nb4 & b5) | (a3 & na6 & b4)) & nb3))) | (nb4 & b2 & b3 & b5));
                long[] array3 = this.extra[6];
                int n3 = k;
                array3[n3] |= otherInits.extra[6][k];
                long[] array4 = this.extra[7];
                int n4 = k;
                array4[n4] |= otherInits.extra[7][k];
                thisHasNulls = (thisHasNulls || this.extra[3][k] != 0L || this.extra[4][k] != 0L || this.extra[5][k] != 0L);
            }
            while (k < copyLimit) {
                this.extra[2][k] = 0L;
                long[] array5 = this.extra[3];
                int n5 = k;
                long b2 = otherInits.extra[3][k];
                long b3;
                long b4;
                long nb4;
                long nb3;
                array5[n5] = (b2 & (nb3 = (~(b3 = otherInits.extra[4][k]) | (nb4 = ~(b4 = otherInits.extra[2][k])))));
                long nb5;
                long b5;
                this.extra[4][k] = ((b3 & (((nb5 = ~b2) & (b5 = otherInits.extra[5][k])) | nb4)) | (b4 & nb5 & ~b5));
                this.extra[5][k] = (((nb3 | nb5) & nb4 & b5) | (b4 & nb3 & nb5));
                long[] array6 = this.extra[6];
                int n6 = k;
                array6[n6] |= otherInits.extra[6][k];
                long[] array7 = this.extra[7];
                int n7 = k;
                array7[n7] |= otherInits.extra[7][k];
                thisHasNulls = (thisHasNulls || this.extra[3][k] != 0L || this.extra[4][k] != 0L || this.extra[5][k] != 0L);
                ++k;
            }
            while (k < resetLimit) {
                long a1 = this.extra[2][k];
                this.extra[2][k] = 0L;
                long[] array8 = this.extra[3];
                int n8 = k;
                long a2 = this.extra[3][k];
                long a3;
                long na4;
                long na3;
                array8[n8] = (a2 & (na3 = (~(a3 = this.extra[4][k]) | (na4 = ~a1))));
                long na5;
                long a4;
                this.extra[4][k] = ((a3 & (((na5 = ~a2) & (a4 = this.extra[5][k])) | na4)) | (a1 & na5 & ~a4));
                this.extra[5][k] = (((na3 | na5) & na4 & a4) | (a1 & na3 & na5));
                thisHasNulls = (thisHasNulls || this.extra[3][k] != 0L || this.extra[4][k] != 0L || this.extra[5][k] != 0L);
                ++k;
            }
        }
        if (thisHasNulls) {
            this.tagBits |= 0x4;
            return this;
        }
        this.tagBits &= 0xFFFFFFFB;
        return this;
    }
    
    static int numberOfEnclosingFields(ReferenceBinding type) {
        int count = 0;
        for (type = type.enclosingType(); type != null; type = type.enclosingType()) {
            count += type.fieldCount();
        }
        return count;
    }
    
    @Override
    public UnconditionalFlowInfo nullInfoLessUnconditionalCopy() {
        if (this == UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        UnconditionalFlowInfo copy = new UnconditionalFlowInfo();
        copy.definiteInits = this.definiteInits;
        copy.potentialInits = this.potentialInits;
        copy.iNBit = -1L;
        copy.iNNBit = -1L;
        copy.tagBits = (this.tagBits & 0xFFFFFFFB);
        copy.tagBits |= 0x40;
        copy.maxFieldCount = this.maxFieldCount;
        if (this.extra != null) {
            copy.extra = new long[8][];
            int length;
            System.arraycopy(this.extra[0], 0, copy.extra[0] = new long[length = this.extra[0].length], 0, length);
            System.arraycopy(this.extra[1], 0, copy.extra[1] = new long[length], 0, length);
            for (int j = 2; j < 8; ++j) {
                copy.extra[j] = new long[length];
            }
            Arrays.fill(copy.extra[6], -1L);
            Arrays.fill(copy.extra[7], -1L);
            return copy;
        }
        return copy;
    }
    
    @Override
    public FlowInfo safeInitsWhenTrue() {
        return this.copy();
    }
    
    @Override
    public FlowInfo setReachMode(int reachMode) {
        if (this == UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        if (reachMode == 0) {
            this.tagBits &= 0xFFFFFFFC;
        }
        else if (reachMode == 2) {
            this.tagBits |= 0x2;
        }
        else {
            if ((this.tagBits & 0x3) == 0x0) {
                this.potentialInits = 0L;
                if (this.extra != null) {
                    for (int i = 0, length = this.extra[0].length; i < length; ++i) {
                        this.extra[1][i] = 0L;
                    }
                }
            }
            this.tagBits |= reachMode;
        }
        return this;
    }
    
    @Override
    public String toString() {
        if (this == UnconditionalFlowInfo.DEAD_END) {
            return "FlowInfo.DEAD_END";
        }
        if ((this.tagBits & 0x4) != 0x0) {
            if (this.extra == null) {
                return "FlowInfo<def: " + this.definiteInits + ", pot: " + this.potentialInits + ", reachable:" + ((this.tagBits & 0x3) == 0x0) + ", null: " + this.nullBit1 + this.nullBit2 + this.nullBit3 + this.nullBit4 + ", incoming: " + this.iNBit + this.iNNBit + ">";
            }
            String def = "FlowInfo<def:[" + this.definiteInits;
            String pot = "], pot:[" + this.potentialInits;
            String nullS = ", null:[" + this.nullBit1 + this.nullBit2 + this.nullBit3 + this.nullBit4;
            int j;
            int ceil2;
            for (j = 0, ceil2 = ((this.extra[0].length > 3) ? 3 : this.extra[0].length); j < ceil2; ++j) {
                def = def + "," + this.extra[0][j];
                pot = pot + "," + this.extra[1][j];
                nullS = nullS + "," + this.extra[2][j] + this.extra[3][j] + this.extra[4][j] + this.extra[5][j] + ", incoming: " + this.extra[6][j] + this.extra[7];
            }
            if (ceil2 < this.extra[0].length) {
                def = def + ",...";
                pot = pot + ",...";
                nullS = nullS + ",...";
            }
            return def + pot + "], reachable:" + ((this.tagBits & 0x3) == 0x0) + nullS + "]>";
        }
        else {
            if (this.extra == null) {
                return "FlowInfo<def: " + this.definiteInits + ", pot: " + this.potentialInits + ", reachable:" + ((this.tagBits & 0x3) == 0x0) + ", no null info>";
            }
            String def = "FlowInfo<def:[" + this.definiteInits;
            String pot = "], pot:[" + this.potentialInits;
            int i;
            int ceil;
            for (i = 0, ceil = ((this.extra[0].length > 3) ? 3 : this.extra[0].length); i < ceil; ++i) {
                def = def + "," + this.extra[0][i];
                pot = pot + "," + this.extra[1][i];
            }
            if (ceil < this.extra[0].length) {
                def = def + ",...";
                pot = pot + ",...";
            }
            return def + pot + "], reachable:" + ((this.tagBits & 0x3) == 0x0) + ", no null info>";
        }
    }
    
    @Override
    public UnconditionalFlowInfo unconditionalCopy() {
        return (UnconditionalFlowInfo)this.copy();
    }
    
    @Override
    public UnconditionalFlowInfo unconditionalFieldLessCopy() {
        UnconditionalFlowInfo copy = new UnconditionalFlowInfo();
        copy.tagBits = this.tagBits;
        copy.maxFieldCount = this.maxFieldCount;
        int limit = this.maxFieldCount;
        if (limit < 64) {
            long mask;
            copy.definiteInits = (this.definiteInits & (mask = ~((1L << limit) - 1L)));
            copy.potentialInits = (this.potentialInits & mask);
            copy.nullBit1 = (this.nullBit1 & mask);
            copy.nullBit2 = (this.nullBit2 & mask);
            copy.nullBit3 = (this.nullBit3 & mask);
            copy.nullBit4 = (this.nullBit4 & mask);
            copy.iNBit = (this.iNBit & mask);
            copy.iNNBit = (this.iNNBit & mask);
        }
        if (this.extra == null) {
            return copy;
        }
        int vectorIndex;
        int length;
        if ((vectorIndex = limit / 64 - 1) >= (length = this.extra[0].length)) {
            return copy;
        }
        copy.extra = new long[8][];
        int copyStart;
        if ((copyStart = vectorIndex + 1) < length) {
            int copyLength = length - copyStart;
            for (int j = 0; j < 8; ++j) {
                System.arraycopy(this.extra[j], copyStart, copy.extra[j] = new long[length], copyStart, copyLength);
            }
        }
        else if (vectorIndex >= 0) {
            copy.createExtraSpace(length);
        }
        if (vectorIndex >= 0) {
            long mask2 = ~((1L << limit % 64) - 1L);
            for (int i = 0; i < 8; ++i) {
                copy.extra[i][vectorIndex] = (this.extra[i][vectorIndex] & mask2);
            }
            return copy;
        }
        return copy;
    }
    
    @Override
    public UnconditionalFlowInfo unconditionalInits() {
        return this;
    }
    
    @Override
    public UnconditionalFlowInfo unconditionalInitsWithoutSideEffect() {
        return this;
    }
    
    @Override
    public UnconditionalFlowInfo mergeDefiniteInitsWith(UnconditionalFlowInfo otherInits) {
        if ((otherInits.tagBits & 0x1) != 0x0 && this != UnconditionalFlowInfo.DEAD_END) {
            return this;
        }
        if ((this.tagBits & 0x1) != 0x0) {
            return (UnconditionalFlowInfo)otherInits.copy();
        }
        this.definiteInits &= otherInits.definiteInits;
        if (this.extra != null) {
            if (otherInits.extra != null) {
                int i = 0;
                int length;
                int otherLength2;
                if ((length = this.extra[0].length) < (otherLength2 = otherInits.extra[0].length)) {
                    for (int j = 0; j < 8; ++j) {
                        System.arraycopy(this.extra[j], 0, this.extra[j] = new long[otherLength2], 0, length);
                    }
                    while (i < length) {
                        long[] array2 = this.extra[0];
                        int n2 = i;
                        array2[n2] &= otherInits.extra[0][i];
                        ++i;
                    }
                    while (i < otherLength2) {
                        this.extra[0][i] = otherInits.extra[0][i];
                        ++i;
                    }
                }
                else {
                    while (i < otherLength2) {
                        long[] array = this.extra[0];
                        int n = i;
                        array[n] &= otherInits.extra[0][i];
                        ++i;
                    }
                }
            }
            else {
                for (int i = 0; i < this.extra[0].length; ++i) {
                    this.extra[0][i] = 0L;
                }
            }
        }
        else if (otherInits.extra != null) {
            int otherLength = otherInits.extra[0].length;
            this.createExtraSpace(otherLength);
            System.arraycopy(otherInits.extra[0], 0, this.extra[0], 0, otherLength);
        }
        return this;
    }
    
    @Override
    public void resetAssignmentInfo(LocalVariableBinding local) {
        this.resetAssignmentInfo(local.id + this.maxFieldCount);
    }
    
    public void resetAssignmentInfo(int position) {
        if (this != UnconditionalFlowInfo.DEAD_END) {
            if (position < 64) {
                long mask2;
                this.definiteInits &= (mask2 = ~(1L << position));
                this.potentialInits &= mask2;
            }
            else {
                int vectorIndex = position / 64 - 1;
                if (this.extra == null || vectorIndex >= this.extra[0].length) {
                    return;
                }
                long[] array = this.extra[0];
                int n = vectorIndex;
                long mask;
                array[n] &= (mask = ~(1L << position % 64));
                long[] array2 = this.extra[1];
                int n2 = vectorIndex;
                array2[n2] &= mask;
            }
        }
    }
    
    private void createExtraSpace(int length) {
        this.extra = new long[8][];
        for (int j = 0; j < 8; ++j) {
            this.extra[j] = new long[length];
        }
        if ((this.tagBits & 0x40) != 0x0) {
            Arrays.fill(this.extra[6], -1L);
            Arrays.fill(this.extra[7], -1L);
        }
    }
    
    public static class AssertionFailedException extends RuntimeException
    {
        private static final long serialVersionUID = 1827352841030089703L;
        
        public AssertionFailedException(String message) {
            super(message);
        }
    }
}
